<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">
    <link href="./static/bootstrap.min.css" rel="stylesheet">
    <title>GUNIFLASK - Debug only</title>
</head>
<body style="margin:1em">
<h2>Debug only</h2>
<div class="alert alert-danger" role="alert">This page can be only viewed under debug mode.</div>
</body>
</html>
