<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">
    <link href="./static/bootstrap.min.css" rel="stylesheet">
    <title>GUNIFLASK - Hello World!</title>
</head>
<body style="margin:1em">
<nav aria-label="breadcrumb">
    <ol class="breadcrumb">
        <li class="breadcrumb-item active" aria-current="page">Home</li>
    </ol>
</nav>
<h2>Hello World!</h2>
<p>Available Links:</p>
<ul>
    <li><a href="./login">Login</a></li>
    <li><a href="./settings">App Settings</a></li>
    <li><a href="../_apidoc" target="_blank">API Documentation</a></li>
</ul>
</body>
</html>
